body, input {
  color: #123;
  font-family: "Gill Sans", sans-serif;
}

div {
  overflow: hidden;
  padding: 1em 0;
  position: relative;
  text-align: center;
}

h1, h2, p, input, a {
  font-weight: 300;
  margin: 0;
}

h1 {
  color: #BDB76B;
  font-size: 3.5em;
}

h2 {
  color: #999;
}

form {
  margin: 0 auto;
  max-width: 50em;
  text-align: center;
}

input {
  border: 0;
  border-radius: 1000px;
  box-shadow: inset 0 0 0 2px #BDB76B;
  display: inline;
  font-size: 1.5em;
  margin-bottom: 1em;
  outline: none;
  padding: .5em 5%;
  width: 55%;
}

form a {
  background: #BDB76B;
  border: 0;
  border-radius: 1000px;
  color: #FFF;
  font-size: 1.25em;
  font-weight: 400;
  padding: .75em 2em;
  text-decoration: none;
  text-transform: uppercase;
  white-space: normal;
}

p {
  font-size: 1.5em;
  line-height: 1.5;
}
