// Package objects contains functionality for working with Object Storage
// object resources. An object is a resource that represents and contains data
// - such as documents, images, and so on. You can also store custom metadata
// with an object.
package objects
