<md-button class="md-button-toggle"
           ng-click="toggle()"
           aria-controls="kubernetes-ui-menu-{{section.name | nospace}}"
           flex layout="row"
           aria-expanded="{{isOpen()}}">
  {{section.name}}
  <span aria-hidden="true" class="md-toggle-icon" ng-class="{'toggled' : isOpen()}"></span>
  <span class="visually-hidden">Toggle {{isOpen()? 'expanded' : 'collapsed'}}</span>
</md-button>
<ul ng-show="isOpen()" id="kubernetes-ui-menu-{{section.name | nospace}}" class="menu-toggle-list">
  <li ng-repeat="page in section.pages">
    <menu-link section="page"></menu-link>
  </li>
</ul>
