.groups {
  font-size: 13px;

  .header {
    line-height: 21px;
    a {
      padding-left: 5px;
      padding-right: 5px;
    }

    .selector-area {

      .filter-area {
        // font-size: 15px;
        // padding-left: 20px;
      }

      .filter-label {

      }

      .filter-text {
        font-size: 13px;
        margin-left: 10px;
      }

      .cancel-button {
        width: 18px;
        height: 18px;
        padding: 0;

        &:focus, &:hover {
          background-color: none !important;
        }
      }

      .cancel-icon {
        width: 15px;
        height: 15px;
        fill: #777777;
      }

    }
  }

  .group-heading {

    .label {
      // font-size: 18px;
    }
  }

  .select-group-by {
    min-width: 110px;
    margin-left: 10px;
    margin-right: 40px;

    .md-select-label {
      padding-top: 6px;
      font-size: 13px;
    }
  }

  .group-item {
    padding-top: 20px;

    .filter-button {
      height: 18px;
      width: 18px;

      .filter-icon {
        width: 18px;
        height: 18px;
        // fill: #777777;
      }

    }
  }

  .icon-area {
    // padding-right: 10px;
    min-width: 34px;
    .group-icon {
      border-radius: 21px;
      width: 21px;
      height: 21px;
    }
  }

  .group-main-area {
    // padding-bottom: 10px;

    .subtype {
      line-height: 21px;
    }
  }

  md-divider {
    margin-top: 40px;
    margin-bottom: 30px;
  }

  .group-name {
    padding-top: 10px;
  }
  .selectFilter {
    padding-top: 7px;
    margin-right: 30px;

    .md-select-label {
      border-bottom: none !important;
      width: 17px;
      min-width: 17px;
      padding-right: 0;
    }
  }

  md-select-menu {
    min-height: 40px;
    max-height: 40px;
  }


  .group-link-area {
    padding-left: 15px;
    padding-bottom: 15px;

    button {
      line-height: 12px;
      // line-height: 12px;
    }
  }

  .group-type-circle {
    width: 21px;
    height: 21px;
  }
  .group-type-circle span {

  }
  // .group-type-area .pod-color
  // {
    //   background-color: rgb(57,73,171);
  // }
  // .group-type-area .replicationController-color
  // {
    //   background-color: rgb(57,73,171);
  // }

  // .group-type-area .service-color
  // {
    //   background-color: rgb(57,73,171);
  // }

  md-select {
    margin-top: 0px;
  }
}
