apiVersion: v1
kind: Config
current-context: kubectl-to-{{ cluster_name }}
preferences: {}
clusters:
- cluster:
    certificate-authority-data: {{ kube_ca_cert|b64encode }}
    server: https://{{ groups['masters'][0] }}:443
  name: {{ cluster_name }}
contexts:
- context:
    cluster: {{ cluster_name }}
    user: kubectl
  name: kubectl-to-{{ cluster_name }}
users:
- name: kubectl
  user:
    token: {{ kubectl_token }}
