#!/bin/bash

# Copyright 2014 The Kubernetes Authors All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This script installs std -race on Travis (see https://code.google.com/p/go/issues/detail?id=6479)

set -o errexit
set -o nounset
set -o pipefail

if [[ "${TRAVIS}" == "true" ]]; then
  GO_VERSION=($(go version))

  if [[ ${GO_VERSION[2]} < "go1.3" ]]; then
    echo "Installing the -race compatible version of the std go library"
    go install -a -race std
  fi
fi
