apiVersion: v1
kind: Config
current-context: scheduler-to-{{ cluster_name }}
preferences: {}
clusters:
- cluster:
    certificate-authority: {{ kube_cert_dir }}/ca.crt
    server: https://{{ groups['masters'][0] }}:443
  name: {{ cluster_name }}
contexts:
- context:
    cluster: {{ cluster_name }}
    user: scheduler
  name: scheduler-to-{{ cluster_name }}
users:
- name: scheduler
  user:
    token: {{ scheduler_token }}
