md-autocomplete {
  background: '{{background-50}}'; }
  md-autocomplete button md-icon path {
    fill: '{{background-600}}'; }
  md-autocomplete button:after {
    background: '{{background-600-0.3}}'; }
  md-autocomplete ul {
    background: '{{background-50}}'; }
    md-autocomplete ul li {
      border-top: 1px solid '{{background-400}}';
      color: '{{background-900}}'; }
      md-autocomplete ul li .highlight {
        color: '{{background-600}}'; }
      md-autocomplete ul li:hover, md-autocomplete ul li.selected {
        background: '{{background-200}}'; }

md-backdrop.md-opaque.md-THEME_NAME-theme {
  background-color: '{{foreground-4-0.5}}'; }

md-bottom-sheet.md-THEME_NAME-theme {
  background-color: '{{background-50}}';
  border-top-color: '{{background-300}}'; }
  md-bottom-sheet.md-THEME_NAME-theme.md-list md-item {
    color: '{{foreground-1}}'; }
  md-bottom-sheet.md-THEME_NAME-theme .md-subheader {
    background-color: '{{background-50}}'; }
  md-bottom-sheet.md-THEME_NAME-theme .md-subheader {
    color: '{{foreground-1}}'; }

md-toolbar .md-button.md-THEME_NAME-theme.md-fab {
  background-color: white; }

.md-button.md-THEME_NAME-theme {
  border-radius: 3px; }
  .md-button.md-THEME_NAME-theme:not([disabled]):hover, .md-button.md-THEME_NAME-theme:not([disabled]):focus {
    background-color: '{{background-500-0.2}}'; }
  .md-button.md-THEME_NAME-theme.md-primary {
    color: '{{primary-color}}'; }
    .md-button.md-THEME_NAME-theme.md-primary.md-raised, .md-button.md-THEME_NAME-theme.md-primary.md-fab {
      color: '{{primary-contrast}}';
      background-color: '{{primary-color}}'; }
      .md-button.md-THEME_NAME-theme.md-primary.md-raised:not([disabled]):hover, .md-button.md-THEME_NAME-theme.md-primary.md-raised:not([disabled]):focus, .md-button.md-THEME_NAME-theme.md-primary.md-fab:not([disabled]):hover, .md-button.md-THEME_NAME-theme.md-primary.md-fab:not([disabled]):focus {
        background-color: '{{primary-600}}'; }
  .md-button.md-THEME_NAME-theme.md-fab {
    border-radius: 50%;
    background-color: '{{accent-color}}';
    color: '{{accent-contrast}}'; }
    .md-button.md-THEME_NAME-theme.md-fab:not([disabled]):hover, .md-button.md-THEME_NAME-theme.md-fab:not([disabled]):focus {
      background-color: '{{accent-A700}}'; }
  .md-button.md-THEME_NAME-theme.md-raised {
    color: '{{background-contrast}}';
    background-color: '{{background-50}}'; }
    .md-button.md-THEME_NAME-theme.md-raised:not([disabled]):hover, .md-button.md-THEME_NAME-theme.md-raised:not([disabled]):focus {
      background-color: '{{background-200}}'; }
  .md-button.md-THEME_NAME-theme.md-warn {
    color: '{{warn-color}}'; }
    .md-button.md-THEME_NAME-theme.md-warn.md-raised, .md-button.md-THEME_NAME-theme.md-warn.md-fab {
      color: '{{warn-contrast}}';
      background-color: '{{warn-color}}'; }
      .md-button.md-THEME_NAME-theme.md-warn.md-raised:not([disabled]):hover, .md-button.md-THEME_NAME-theme.md-warn.md-raised:not([disabled]):focus, .md-button.md-THEME_NAME-theme.md-warn.md-fab:not([disabled]):hover, .md-button.md-THEME_NAME-theme.md-warn.md-fab:not([disabled]):focus {
        background-color: '{{warn-700}}'; }
  .md-button.md-THEME_NAME-theme.md-accent {
    color: '{{accent-color}}'; }
    .md-button.md-THEME_NAME-theme.md-accent.md-raised, .md-button.md-THEME_NAME-theme.md-accent.md-fab {
      color: '{{accent-contrast}}';
      background-color: '{{accent-color}}'; }
      .md-button.md-THEME_NAME-theme.md-accent.md-raised:not([disabled]):hover, .md-button.md-THEME_NAME-theme.md-accent.md-raised:not([disabled]):focus, .md-button.md-THEME_NAME-theme.md-accent.md-fab:not([disabled]):hover, .md-button.md-THEME_NAME-theme.md-accent.md-fab:not([disabled]):focus {
        background-color: '{{accent-700}}'; }
  .md-button.md-THEME_NAME-theme[disabled], .md-button.md-THEME_NAME-theme.md-raised[disabled], .md-button.md-THEME_NAME-theme.md-fab[disabled] {
    color: '{{foreground-3}}';
    background-color: transparent;
    cursor: not-allowed; }

md-card.md-THEME_NAME-theme {
  border-radius: 2px; }
  md-card.md-THEME_NAME-theme .md-card-image {
    border-radius: 2px 2px 0 0; }

md-checkbox.md-THEME_NAME-theme .md-ripple {
  color: '{{accent-600}}'; }
md-checkbox.md-THEME_NAME-theme.md-checked .md-ripple {
  color: '{{background-600}}'; }
md-checkbox.md-THEME_NAME-theme .md-icon {
  border-color: '{{foreground-2}}'; }
md-checkbox.md-THEME_NAME-theme.md-checked .md-icon {
  background-color: '{{accent-color-0.87}}'; }
md-checkbox.md-THEME_NAME-theme.md-checked .md-icon:after {
  border-color: '{{background-200}}'; }
md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary .md-ripple {
  color: '{{primary-600}}'; }
md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-ripple {
  color: '{{background-600}}'; }
md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary .md-icon {
  border-color: '{{foreground-2}}'; }
md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-icon {
  background-color: '{{primary-color-0.87}}'; }
md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-icon:after {
  border-color: '{{background-200}}'; }
md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn .md-ripple {
  color: '{{warn-600}}'; }
md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn .md-icon {
  border-color: '{{foreground-2}}'; }
md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked .md-icon {
  background-color: '{{warn-color-0.87}}'; }
md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked .md-icon:after {
  border-color: '{{background-200}}'; }
md-checkbox.md-THEME_NAME-theme[disabled] .md-icon {
  border-color: '{{foreground-3}}'; }
md-checkbox.md-THEME_NAME-theme[disabled].md-checked .md-icon {
  background-color: '{{foreground-3}}'; }

md-content.md-THEME_NAME-theme {
  background-color: '{{background-hue-3}}'; }

md-dialog.md-THEME_NAME-theme {
  border-radius: 4px;
  background-color: '{{background-hue-3}}'; }
  md-dialog.md-THEME_NAME-theme.md-content-overflow .md-actions {
    border-top-color: '{{foreground-4}}'; }

md-divider.md-THEME_NAME-theme {
  border-top-color: '{{foreground-4}}'; }

md-icon.md-THEME_NAME-theme.md-primary {
  color: '{{primary-color}}'; }
md-icon.md-THEME_NAME-theme.md-accent {
  color: '{{accent-color}}'; }
md-icon.md-THEME_NAME-theme.md-warn {
  color: '{{warn-color}}'; }
md-icon.md-THEME_NAME-theme.md-danger {
  color: '{{danger-color}}'; }

md-input-container.md-THEME_NAME-theme .md-input {
  color: '{{foreground-1}}';
  border-color: '{{foreground-4}}';
  text-shadow: '{{foreground-shadow}}'; }
  md-input-container.md-THEME_NAME-theme .md-input::-webkit-input-placeholder, md-input-container.md-THEME_NAME-theme .md-input::-moz-placeholder, md-input-container.md-THEME_NAME-theme .md-input:-moz-placeholder, md-input-container.md-THEME_NAME-theme .md-input:-ms-input-placeholder {
    color: '{{foreground-3}}'; }
md-input-container.md-THEME_NAME-theme label, md-input-container.md-THEME_NAME-theme .md-placeholder {
  text-shadow: '{{foreground-shadow}}';
  color: '{{foreground-3}}'; }
md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-has-value label {
  color: '{{foreground-2}}'; }
md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-focused .md-input {
  border-color: '{{primary-500}}'; }
md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-focused label {
  color: '{{primary-500}}'; }
md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-focused.md-accent .md-input {
  border-color: '{{accent-500}}'; }
md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-focused.md-accent label {
  color: '{{accent-500}}'; }
md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-focused.md-warn .md-input {
  border-color: '{{warn-500}}'; }
md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-focused.md-warn label {
  color: '{{warn-500}}'; }
md-input-container.md-THEME_NAME-theme.md-input-invalid .md-input {
  border-color: '{{warn-500}}'; }
md-input-container.md-THEME_NAME-theme.md-input-invalid label {
  color: '{{warn-500}}'; }
md-input-container.md-THEME_NAME-theme.md-input-invalid ng-message, md-input-container.md-THEME_NAME-theme.md-input-invalid data-ng-message, md-input-container.md-THEME_NAME-theme.md-input-invalid x-ng-message, md-input-container.md-THEME_NAME-theme.md-input-invalid [ng-message], md-input-container.md-THEME_NAME-theme.md-input-invalid [data-ng-message], md-input-container.md-THEME_NAME-theme.md-input-invalid [x-ng-message], md-input-container.md-THEME_NAME-theme.md-input-invalid .md-char-counter {
  color: '{{warn-500}}'; }
md-input-container.md-THEME_NAME-theme .md-input[disabled], [disabled] md-input-container.md-THEME_NAME-theme .md-input {
  border-bottom-color: transparent;
  color: '{{foreground-3}}';
  background-image: linear-gradient(to right, '{{foreground-4}}' 0%, '{{foreground-4}}' 33%, transparent 0%);
  background-image: -ms-linear-gradient(left, transparent 0%, '{{foreground-4}}' 100%); }

md-progress-circular.md-THEME_NAME-theme {
  background-color: transparent; }
  md-progress-circular.md-THEME_NAME-theme .md-inner .md-gap {
    border-top-color: '{{primary-color}}';
    border-bottom-color: '{{primary-color}}'; }
  md-progress-circular.md-THEME_NAME-theme .md-inner .md-left .md-half-circle, md-progress-circular.md-THEME_NAME-theme .md-inner .md-right .md-half-circle {
    border-top-color: '{{primary-color}}'; }
  md-progress-circular.md-THEME_NAME-theme .md-inner .md-right .md-half-circle {
    border-right-color: '{{primary-color}}'; }
  md-progress-circular.md-THEME_NAME-theme .md-inner .md-left .md-half-circle {
    border-left-color: '{{primary-color}}'; }
  md-progress-circular.md-THEME_NAME-theme.md-warn .md-inner .md-gap {
    border-top-color: '{{warn-color}}';
    border-bottom-color: '{{warn-color}}'; }
  md-progress-circular.md-THEME_NAME-theme.md-warn .md-inner .md-left .md-half-circle, md-progress-circular.md-THEME_NAME-theme.md-warn .md-inner .md-right .md-half-circle {
    border-top-color: '{{warn-color}}'; }
  md-progress-circular.md-THEME_NAME-theme.md-warn .md-inner .md-right .md-half-circle {
    border-right-color: '{{warn-color}}'; }
  md-progress-circular.md-THEME_NAME-theme.md-warn .md-inner .md-left .md-half-circle {
    border-left-color: '{{warn-color}}'; }
  md-progress-circular.md-THEME_NAME-theme.md-accent .md-inner .md-gap {
    border-top-color: '{{accent-color}}';
    border-bottom-color: '{{accent-color}}'; }
  md-progress-circular.md-THEME_NAME-theme.md-accent .md-inner .md-left .md-half-circle, md-progress-circular.md-THEME_NAME-theme.md-accent .md-inner .md-right .md-half-circle {
    border-top-color: '{{accent-color}}'; }
  md-progress-circular.md-THEME_NAME-theme.md-accent .md-inner .md-right .md-half-circle {
    border-right-color: '{{accent-color}}'; }
  md-progress-circular.md-THEME_NAME-theme.md-accent .md-inner .md-left .md-half-circle {
    border-left-color: '{{accent-color}}'; }

md-progress-linear.md-THEME_NAME-theme .md-container {
  background-color: '{{primary-100}}'; }
md-progress-linear.md-THEME_NAME-theme .md-bar {
  background-color: '{{primary-color}}'; }
md-progress-linear.md-THEME_NAME-theme.md-warn .md-container {
  background-color: '{{warn-100}}'; }
md-progress-linear.md-THEME_NAME-theme.md-warn .md-bar {
  background-color: '{{warn-color}}'; }
md-progress-linear.md-THEME_NAME-theme.md-accent .md-container {
  background-color: '{{accent-100}}'; }
md-progress-linear.md-THEME_NAME-theme.md-accent .md-bar {
  background-color: '{{accent-color}}'; }
md-progress-linear.md-THEME_NAME-theme[md-mode=buffer].md-warn .md-bar1 {
  background-color: '{{warn-100}}'; }
md-progress-linear.md-THEME_NAME-theme[md-mode=buffer].md-warn .md-dashed:before {
  background: radial-gradient('{{warn-100}}' 0%, '{{warn-100}}' 16%, transparent 42%); }
md-progress-linear.md-THEME_NAME-theme[md-mode=buffer].md-accent .md-bar1 {
  background-color: '{{accent-100}}'; }
md-progress-linear.md-THEME_NAME-theme[md-mode=buffer].md-accent .md-dashed:before {
  background: radial-gradient('{{accent-100}}' 0%, '{{accent-100}}' 16%, transparent 42%); }

md-radio-button.md-THEME_NAME-theme .md-off {
  border-color: '{{foreground-2}}'; }
md-radio-button.md-THEME_NAME-theme .md-on {
  background-color: '{{accent-color-0.87}}'; }
md-radio-button.md-THEME_NAME-theme.md-checked .md-off {
  border-color: '{{accent-color-0.87}}'; }
md-radio-button.md-THEME_NAME-theme.md-checked .md-ink-ripple {
  color: '{{accent-color-0.87}}'; }
md-radio-button.md-THEME_NAME-theme .md-container .md-ripple {
  color: '{{accent-600}}'; }
md-radio-button.md-THEME_NAME-theme:not([disabled]).md-primary .md-on {
  background-color: '{{primary-color-0.87}}'; }
md-radio-button.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-off {
  border-color: '{{primary-color-0.87}}'; }
md-radio-button.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-ink-ripple {
  color: '{{primary-color-0.87}}'; }
md-radio-button.md-THEME_NAME-theme:not([disabled]).md-primary .md-container .md-ripple {
  color: '{{primary-600}}'; }
md-radio-button.md-THEME_NAME-theme:not([disabled]).md-warn .md-on {
  background-color: '{{warn-color-0.87}}'; }
md-radio-button.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked .md-off {
  border-color: '{{warn-color-0.87}}'; }
md-radio-button.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked .md-ink-ripple {
  color: '{{warn-color-0.87}}'; }
md-radio-button.md-THEME_NAME-theme:not([disabled]).md-warn .md-container .md-ripple {
  color: '{{warn-600}}'; }
md-radio-button.md-THEME_NAME-theme[disabled] .md-container .md-off {
  border-color: '{{foreground-3}}'; }
md-radio-button.md-THEME_NAME-theme[disabled] .md-container .md-on {
  border-color: '{{foreground-3}}'; }

md-radio-group.md-THEME_NAME-theme:focus:not(:empty) {
  border-color: '{{foreground-1}}'; }

md-select.md-THEME_NAME-theme:focus .md-select-label {
  border-bottom-color: '{{primary-color}}';
  color: '{{ foreground-1 }}'; }
  md-select.md-THEME_NAME-theme:focus .md-select-label.md-placeholder {
    color: '{{ foreground-1 }}'; }
md-select.md-THEME_NAME-theme:focus.md-accent .md-select-label {
  border-bottom-color: '{{accent-color}}'; }
md-select.md-THEME_NAME-theme:focus.md-warn .md-select-label {
  border-bottom-color: '{{warn-color}}'; }
md-select.md-THEME_NAME-theme .md-select-label {
  border-bottom-color: '{{foreground-4}}'; }
  md-select.md-THEME_NAME-theme .md-select-label.md-placeholder {
    color: '{{foreground-3}}'; }

md-select-menu.md-THEME_NAME-theme md-optgroup {
  color: '{{foreground-2}}'; }
  md-select-menu.md-THEME_NAME-theme md-optgroup md-option {
    color: '{{foreground-1}}'; }
md-select-menu.md-THEME_NAME-theme md-option[selected] {
  background-color: '{{primary-50}}'; }
  md-select-menu.md-THEME_NAME-theme md-option[selected]:focus {
    background-color: '{{primary-100}}'; }
  md-select-menu.md-THEME_NAME-theme md-option[selected].md-accent {
    background-color: '{{accent-50}}'; }
    md-select-menu.md-THEME_NAME-theme md-option[selected].md-accent:focus {
      background-color: '{{accent-100}}'; }
md-select-menu.md-THEME_NAME-theme md-option:focus:not([selected]) {
  background: '{{background-200}}'; }

md-sidenav.md-THEME_NAME-theme {
  background-color: '{{background-hue-3}}'; }

md-slider.md-THEME_NAME-theme .md-track {
  background-color: '{{foreground-3}}'; }
md-slider.md-THEME_NAME-theme .md-track-ticks {
  background-color: '{{foreground-4}}'; }
md-slider.md-THEME_NAME-theme .md-focus-thumb {
  background-color: '{{foreground-2}}'; }
md-slider.md-THEME_NAME-theme .md-focus-ring {
  border-color: '{{foreground-4}}'; }
md-slider.md-THEME_NAME-theme .md-disabled-thumb {
  border-color: '{{background-hue-3}}'; }
md-slider.md-THEME_NAME-theme.md-min .md-thumb:after {
  background-color: '{{background-hue-3}}'; }
md-slider.md-THEME_NAME-theme .md-track.md-track-fill {
  background-color: '{{accent-color}}'; }
md-slider.md-THEME_NAME-theme .md-thumb:after {
  border-color: '{{accent-color}}';
  background-color: '{{accent-color}}'; }
md-slider.md-THEME_NAME-theme .md-sign {
  background-color: '{{accent-color}}'; }
  md-slider.md-THEME_NAME-theme .md-sign:after {
    border-top-color: '{{accent-color}}'; }
md-slider.md-THEME_NAME-theme .md-thumb-text {
  color: '{{accent-contrast}}'; }
md-slider.md-THEME_NAME-theme.md-warn .md-track.md-track-fill {
  background-color: '{{warn-color}}'; }
md-slider.md-THEME_NAME-theme.md-warn .md-thumb:after {
  border-color: '{{warn-color}}';
  background-color: '{{warn-color}}'; }
md-slider.md-THEME_NAME-theme.md-warn .md-sign {
  background-color: '{{warn-color}}'; }
  md-slider.md-THEME_NAME-theme.md-warn .md-sign:after {
    border-top-color: '{{warn-color}}'; }
md-slider.md-THEME_NAME-theme.md-warn .md-thumb-text {
  color: '{{warn-contrast}}'; }
md-slider.md-THEME_NAME-theme.md-primary .md-track.md-track-fill {
  background-color: '{{primary-color}}'; }
md-slider.md-THEME_NAME-theme.md-primary .md-thumb:after {
  border-color: '{{primary-color}}';
  background-color: '{{primary-color}}'; }
md-slider.md-THEME_NAME-theme.md-primary .md-sign {
  background-color: '{{primary-color}}'; }
  md-slider.md-THEME_NAME-theme.md-primary .md-sign:after {
    border-top-color: '{{primary-color}}'; }
md-slider.md-THEME_NAME-theme.md-primary .md-thumb-text {
  color: '{{primary-contrast}}'; }
md-slider.md-THEME_NAME-theme[disabled] .md-thumb:after {
  border-color: '{{foreground-3}}'; }
md-slider.md-THEME_NAME-theme[disabled]:not(.md-min) .md-thumb:after {
  background-color: '{{foreground-3}}'; }

.md-subheader.md-THEME_NAME-theme {
  color: '{{ foreground-2-0.23 }}';
  background-color: '{{background-hue-3}}'; }
  .md-subheader.md-THEME_NAME-theme.md-primary {
    color: '{{primary-color}}'; }
  .md-subheader.md-THEME_NAME-theme.md-accent {
    color: '{{accent-color}}'; }
  .md-subheader.md-THEME_NAME-theme.md-warn {
    color: '{{warn-color}}'; }

md-switch.md-THEME_NAME-theme .md-thumb {
  background-color: '{{background-50}}'; }
md-switch.md-THEME_NAME-theme .md-bar {
  background-color: '{{background-500}}'; }
md-switch.md-THEME_NAME-theme.md-checked .md-thumb {
  background-color: '{{accent-color}}'; }
md-switch.md-THEME_NAME-theme.md-checked .md-bar {
  background-color: '{{accent-color-0.5}}'; }
md-switch.md-THEME_NAME-theme.md-checked.md-primary .md-thumb {
  background-color: '{{primary-color}}'; }
md-switch.md-THEME_NAME-theme.md-checked.md-primary .md-bar {
  background-color: '{{primary-color-0.5}}'; }
md-switch.md-THEME_NAME-theme.md-checked.md-warn .md-thumb {
  background-color: '{{warn-color}}'; }
md-switch.md-THEME_NAME-theme.md-checked.md-warn .md-bar {
  background-color: '{{warn-color-0.5}}'; }
md-switch.md-THEME_NAME-theme[disabled] .md-thumb {
  background-color: '{{background-400}}'; }
md-switch.md-THEME_NAME-theme[disabled] .md-bar {
  background-color: '{{foreground-4}}'; }
md-switch.md-THEME_NAME-theme:focus .md-label:not(:empty) {
  border-color: '{{foreground-1}}';
  border-style: dotted; }

md-tabs.md-THEME_NAME-theme .md-header {
  background-color: transparent; }
md-tabs.md-THEME_NAME-theme .md-paginator md-icon {
  color: '{{primary-color}}'; }
md-tabs.md-THEME_NAME-theme.md-accent .md-header {
  background-color: '{{accent-color}}'; }
md-tabs.md-THEME_NAME-theme.md-accent md-tab:not([disabled]) {
  color: '{{accent-100}}'; }
  md-tabs.md-THEME_NAME-theme.md-accent md-tab:not([disabled]).active {
    color: '{{accent-contrast}}'; }
md-tabs.md-THEME_NAME-theme.md-primary .md-header {
  background-color: '{{primary-color}}'; }
md-tabs.md-THEME_NAME-theme.md-primary md-tab:not([disabled]) {
  color: '{{primary-100}}'; }
  md-tabs.md-THEME_NAME-theme.md-primary md-tab:not([disabled]).active {
    color: '{{primary-contrast}}'; }
md-tabs.md-THEME_NAME-theme.md-primary md-tab {
  color: '{{primary-100}}'; }
  md-tabs.md-THEME_NAME-theme.md-primary md-tab[disabled] {
    color: '{{foreground-3}}'; }
  md-tabs.md-THEME_NAME-theme.md-primary md-tab:focus {
    color: '{{primary-contrast}}';
    background-color: '{{primary-contrast-0.1}}'; }
  md-tabs.md-THEME_NAME-theme.md-primary md-tab.active {
    color: '{{primary-contrast}}'; }
  md-tabs.md-THEME_NAME-theme.md-primary md-tab .md-ripple-container {
    color: '{{primary-contrast}}'; }
md-tabs.md-THEME_NAME-theme.md-warn .md-header {
  background-color: '{{warn-color}}'; }
md-tabs.md-THEME_NAME-theme.md-warn md-tab:not([disabled]) {
  color: '{{warn-100}}'; }
  md-tabs.md-THEME_NAME-theme.md-warn md-tab:not([disabled]).active {
    color: '{{warn-contrast}}'; }
md-tabs.md-THEME_NAME-theme md-tabs-ink-bar {
  color: '{{accent-color}}';
  background: '{{accent-color}}'; }
md-tabs.md-THEME_NAME-theme md-tab {
  color: '{{foreground-2}}'; }
  md-tabs.md-THEME_NAME-theme md-tab[disabled] {
    color: '{{foreground-3}}'; }
  md-tabs.md-THEME_NAME-theme md-tab:focus {
    color: '{{foreground-1}}'; }
  md-tabs.md-THEME_NAME-theme md-tab.active {
    color: '{{primary-color}}'; }
  md-tabs.md-THEME_NAME-theme md-tab .md-ripple-container {
    color: '{{accent-100}}'; }

md-input-group.md-THEME_NAME-theme input, md-input-group.md-THEME_NAME-theme textarea {
  text-shadow: '{{foreground-shadow}}'; }
  md-input-group.md-THEME_NAME-theme input::-webkit-input-placeholder, md-input-group.md-THEME_NAME-theme input::-moz-placeholder, md-input-group.md-THEME_NAME-theme input:-moz-placeholder, md-input-group.md-THEME_NAME-theme input:-ms-input-placeholder, md-input-group.md-THEME_NAME-theme textarea::-webkit-input-placeholder, md-input-group.md-THEME_NAME-theme textarea::-moz-placeholder, md-input-group.md-THEME_NAME-theme textarea:-moz-placeholder, md-input-group.md-THEME_NAME-theme textarea:-ms-input-placeholder {
    color: '{{foreground-3}}'; }
md-input-group.md-THEME_NAME-theme label {
  text-shadow: '{{foreground-shadow}}';
  color: '{{foreground-3}}'; }
md-input-group.md-THEME_NAME-theme input, md-input-group.md-THEME_NAME-theme textarea {
  color: '{{foreground-1}}';
  border-color: '{{foreground-4}}'; }
md-input-group.md-THEME_NAME-theme.md-input-focused input, md-input-group.md-THEME_NAME-theme.md-input-focused textarea {
  border-color: '{{primary-500}}'; }
md-input-group.md-THEME_NAME-theme.md-input-focused label {
  color: '{{primary-500}}'; }
md-input-group.md-THEME_NAME-theme.md-input-focused.md-accent input, md-input-group.md-THEME_NAME-theme.md-input-focused.md-accent textarea {
  border-color: '{{accent-500}}'; }
md-input-group.md-THEME_NAME-theme.md-input-focused.md-accent label {
  color: '{{accent-500}}'; }
md-input-group.md-THEME_NAME-theme.md-input-has-value:not(.md-input-focused) label {
  color: '{{foreground-2}}'; }
md-input-group.md-THEME_NAME-theme .md-input[disabled] {
  border-bottom-color: '{{foreground-4}}';
  color: '{{foreground-3}}'; }

md-toast.md-THEME_NAME-theme {
  background-color: '{{foreground-1}}';
  color: '{{background-50}}'; }
  md-toast.md-THEME_NAME-theme .md-button {
    color: '{{background-50}}'; }
    md-toast.md-THEME_NAME-theme .md-button.md-highlight {
      color: '{{primary-A200}}'; }
      md-toast.md-THEME_NAME-theme .md-button.md-highlight.md-accent {
        color: '{{accent-A200}}'; }
      md-toast.md-THEME_NAME-theme .md-button.md-highlight.md-warn {
        color: '{{warn-A200}}'; }

md-toolbar.md-THEME_NAME-theme {
  background-color: '{{primary-color}}';
  color: '{{primary-contrast}}'; }
  md-toolbar.md-THEME_NAME-theme .md-button {
    color: '{{primary-contrast}}'; }
  md-toolbar.md-THEME_NAME-theme.md-accent {
    background-color: '{{accent-color}}';
    color: '{{accent-contrast}}'; }
  md-toolbar.md-THEME_NAME-theme.md-warn {
    background-color: '{{warn-color}}';
    color: '{{warn-contrast}}'; }

md-tooltip.md-THEME_NAME-theme {
  color: '{{background-A100}}'; }
  md-tooltip.md-THEME_NAME-theme .md-background {
    background-color: '{{foreground-2}}'; }
