// Package stackresources provides operations for working with stack resources.
// A resource is a template artifact that represents some component of your
// desired architecture (a Cloud Server, a group of scaled Cloud Servers, a load
// balancer, some configuration management system, and so forth).
package stackresources
