// Copyright 2012 Gary Burd
//
// Licensed under the Apache License, Version 2.0 (the "License"): you may
// not use this file except in compliance with the License. You may obtain
// a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

package redis

import (
	"bufio"
	"net"
	"time"
)

func SetNowFunc(f func() time.Time) {
	nowFunc = f
}

type nopCloser struct{ net.Conn }

func (nopCloser) Close() error { return nil }

// NewConnBufio is a hook for tests.
func NewConnBufio(rw bufio.ReadWriter) Conn {
	return &conn{br: rw.Reader, bw: rw.Writer, conn: nopCloser{}}
}

var (
	ErrNegativeInt = errNegativeInt
)
